﻿namespace TabsStudioMarker
{
    public class Marker : TabsStudioExt.ITabsStudioAddin, TabsStudioExt.IStyler
    {
        public Marker()
        {
            highlightMenuItem = new System.Windows.Controls.MenuItem();
            highlightMenuItem.Header = "Highlight";
            highlightMenuItem.Click += OnHighlight;

            removeHighlightingMenuItem = new System.Windows.Controls.MenuItem();
            removeHighlightingMenuItem.Header = "Remove highlighting";
            removeHighlightingMenuItem.Click += OnRemoveHighlighting;

            removeAllHighlightingMenuItem = new System.Windows.Controls.MenuItem();
            removeAllHighlightingMenuItem.Header = "Remove all highlighting";
            removeAllHighlightingMenuItem.Click += OnRemoveAllHighlighting;

            closeAllButHighlightedMenuItem = new System.Windows.Controls.MenuItem();
            closeAllButHighlightedMenuItem.Header = "Close all but highlighted";
            closeAllButHighlightedMenuItem.Click += OnCloseAllButHighlighted;
        }

        public void OnConnection(TabsStudioExt.ITabsStudioEngine engine_, EnvDTE80.DTE2 dte)
        {
            engine = engine_;
            engine.TabCreated += OnTabCreated;
            engine.TabDestroyed += OnTabDestroyed;
            engine.OpeningTabExtensionContextMenu += OpeningTabExtensionContextMenu;
            engine.OpeningTabContextMenu += OpeningTabContextMenu;

            enumWindowFrames = new EnumWindowFrames(dte);

            if (dte.Version.Equals("10.0"))
                stateSaver = new HighlightStateSaver(engine);

            UpdatePresentationStyle();
        }

        public void OnDisconnection()
        {
            engine.TabCreated -= OnTabCreated;
            engine.TabDestroyed -= OnTabDestroyed;
            engine.OpeningTabExtensionContextMenu -= OpeningTabExtensionContextMenu;
            engine.OpeningTabContextMenu -= OpeningTabContextMenu;

            if (stateSaver != null)
                stateSaver.Close();
        }

        public string GetNamespacesForResourceDictionary()
        {
            return "xmlns:TabsStudioMarker=\"clr-namespace:TabsStudioMarker;assembly=Marker\"";
        }

        private void OnTabCreated(object sender, TabsStudioExt.TabEventArgs e)
        {
            e.Tab.TabItem.MouseUp += OnClick;

            if (stateSaver != null && stateSaver.IsHighlighted(e.Tab.TabExtensions[0].Window))
                Properties.SetIsHighlighted(e.Tab.TabItem, true);
        }

        private void OnTabDestroyed(object sender, TabsStudioExt.TabEventArgs e)
        {
            e.Tab.TabItem.MouseUp -= OnClick;            
        }

        private void OpeningTabExtensionContextMenu(object sender, TabsStudioExt.OpeningTabExtensionContextMenuEventArgs e)
        {
            storedTab = e.Tab;
            storedTabs = e.Tabs;
            PlaceMenuItem(e.ContextMenu);
        }

        private void OpeningTabContextMenu(object sender, TabsStudioExt.OpeningTabContextMenuEventArgs e)
        {
            storedTab = e.Tab;
            storedTabs = e.Tabs;
            PlaceMenuItem(e.ContextMenu);
        }

        private void PlaceMenuItem(System.Windows.Controls.ContextMenu menu)
        {
            foreach (object item in menu.Items)
            {
                if (item is System.Windows.Controls.Control)
                {
                    // Only add our menu items when SetColor is present.
                    if ((item as System.Windows.Controls.Control).Name.Equals("SetColor"))
                    {
                        menu.Items.Insert(menu.Items.IndexOf(item), Properties.GetIsHighlighted(storedTab.TabItem) ? 
                            removeHighlightingMenuItem : highlightMenuItem);
                        menu.Items.Insert(menu.Items.IndexOf(item), removeAllHighlightingMenuItem);
                        menu.Items.Insert(menu.Items.IndexOf(item), closeAllButHighlightedMenuItem);
                        menu.Items.Insert(menu.Items.IndexOf(item), new System.Windows.Controls.Separator());
                        break;
                    }
                }
            }
        }

        private void OnClick(object sender, System.Windows.Input.MouseButtonEventArgs e)
        {
            if ((System.Windows.Forms.Control.ModifierKeys & System.Windows.Forms.Keys.Control) != 0)
            {
                System.Windows.UIElement tabItem = sender as System.Windows.Controls.TabItem;
                Properties.SetIsHighlighted(tabItem, !Properties.GetIsHighlighted(tabItem));
            }
        }

        private void OnHighlight(object sender, System.Windows.RoutedEventArgs e)
        {
            Properties.SetIsHighlighted(storedTab.TabItem, true);
        }

        private void OnRemoveHighlighting(object sender, System.Windows.RoutedEventArgs e)
        {
            Properties.SetIsHighlighted(storedTab.TabItem, false);
        }

        private void OnRemoveAllHighlighting(object sender, System.Windows.RoutedEventArgs e)
        {
            foreach (TabsStudioExt.Tab tab in storedTabs.TabList)
                Properties.SetIsHighlighted(tab.TabItem, false);
        }

        private void OnCloseAllButHighlighted(object sender, System.Windows.RoutedEventArgs e)
        {
            System.Collections.Generic.List<EnvDTE.Window> windows = new System.Collections.Generic.List<EnvDTE.Window>();
            foreach (TabsStudioExt.Tab tab in storedTabs.TabList)
            {
                if (!Properties.GetIsHighlighted(tab.TabItem))
                {
                    foreach (TabsStudioExt.TabExtension ext in tab.TabExtensions)
                        windows.Add(ext.Window);
                }
            }

            System.Collections.Generic.List<WindowFrame> frames = enumWindowFrames.GetWindowFrames();
            foreach (EnvDTE.Window w in windows)
            {
                foreach (WindowFrame frame in frames)
                {
                    if (frame.w == w)
                    {
                        bool userCancelled = frame.frame.CloseFrame(
                            (uint)Microsoft.VisualStudio.Shell.Interop.__FRAMECLOSE.FRAMECLOSE_PromptSave) !=
                                Microsoft.VisualStudio.VSConstants.S_OK;
                        if (userCancelled)
                            return;
                        break;
                    }
                }
            }
        }

        private void UpdatePresentationStyle()
        {
            TabsStudioExt.Presentation presentation = new TabsStudioExt.Presentation("", "");
            {
                TabsStudioExt.PresentationStyle style = new TabsStudioExt.PresentationStyle();
                style.Triggers.Add(LoadString("TabInternalsHighlighting.xml"));
                presentation.Styles.Add("TabsStudio:TabInternals", style);
            }
            engine.UpdatePresentationStyles("Marker", presentation);
        }

        private static string LoadString(string resource)
        {
            using (System.IO.Stream stream = System.Reflection.Assembly.GetExecutingAssembly().GetManifestResourceStream(
                "Marker." + resource))
            {
                using (System.IO.StreamReader reader = new System.IO.StreamReader(stream))
                {
                    return reader.ReadToEnd();
                }
            }
        }

        private TabsStudioExt.ITabsStudioEngine engine;
        private TabsStudioExt.Tab storedTab;
        private TabsStudioExt.Tabs storedTabs;
        private System.Windows.Controls.MenuItem highlightMenuItem, removeHighlightingMenuItem, removeAllHighlightingMenuItem;
        private System.Windows.Controls.MenuItem closeAllButHighlightedMenuItem;
        private EnumWindowFrames enumWindowFrames;
        private HighlightStateSaver stateSaver;
    }
}